@@ -80,7 +80,23 @@ class Bunch(dict):
     """
     def __init__(self, **kw):
         dict.__init__(self, kw)
-        self.__dict__ = self
+        self.__dict__.update(kw)
+
+    def __setitem__(self,key,value):
+        dict.__setitem__(self,key,value)
+        self.__dict__[key] = value
+
+    def __setattr__(self,key,value):
+        dict.__setitem__(self,key,value)
+        self.__dict__[key] = value
+
+    def __delitem__(self,key):
+        dict.__delitem__(self,key)
+        del self.__dict__[key]
+
+    def __delattr__(self,key):
+        dict.__delitem__(self,key)
+        del self.__dict__[key]
 
     def __str__(self):
         state = ["%s=%r" % (attribute, value)
@@ -630,10 +646,15 @@ def grib_set_double_array(gribid,key,inarray):
         a = _internal.new_doubleArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         for i in range(nval):
             _internal.doubleArray_setitem(a,i,inarray[i])
+
         GRIB_CHECK(_internal.grib_c_set_real8_array(gribid,key,a,s))
 
+        _internal.delete_doubleArray(a)
+        _internal.delete_intp(s)
+
 @require(gribid=int,key=str)
 def grib_get_double_array(gribid,key):
     """
@@ -657,10 +678,16 @@ def grib_get_double_array(gribid,key):
         a = _internal.new_doubleArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         GRIB_CHECK(_internal.grib_c_get_real8_array(gribid,key,a,s))
+
         result = array("d")
         for i in range(nval):
             result.append(_internal.doubleArray_getitem(a,i))
+
+        _internal.delete_doubleArray(a)
+        _internal.delete_intp(s)
+
         return result
 
 @require(gribid=int,key=str)
@@ -686,10 +713,15 @@ def grib_set_long_array(gribid,key,value):
         a = _internal.new_longArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         for i in range(nval):
             _internal.longArray_setitem(a,i,inarray[i])
+
         GRIB_CHECK(_internal.grib_c_set_long_array(gribid,key,a,s))
 
+        _internal.delete_longArray(a)
+        _internal.delete_intp(s)
+
 @require(gribid=int,key=str)
 def grib_get_long_array(gribid,key):
     """
@@ -713,10 +745,16 @@ def grib_get_long_array(gribid,key):
         a = _internal.new_longArray(nval)
         s = _internal.intp()
         s.assign(nval)
+
         GRIB_CHECK(_internal.grib_c_get_long_array(gribid,key,a,s))
+
         result = array("l")
         for i in range(nval):
             result.append(_internal.longArray_getitem(a,i))
+
+        _internal.delete_longArray(a)
+        _internal.delete_intp(s)
+
         return result
 
 def grib_multi_new():
@@ -835,10 +873,16 @@ def grib_index_get_long(indexid,key):
     a = _internal.new_longArray(nval)
     s = _internal.intp()
     s.assign(nval)
+
     GRIB_CHECK(_internal.grib_c_index_get_long(indexid,key,a,s))
+
     result = []
     for i in range(nval):
         result.append(_internal.longArray_getitem(a,i))
+
+    _internal.delete_longArray(a)
+    _internal.delete_intp(s)
+
     return tuple(result)
 
 def grib_index_get_string(indexid,key):
@@ -890,10 +934,16 @@ def grib_index_get_double(iid,key):
     a = _internal.new_doubleArray(nval)
     s = _internal.intp()
     s.assign(nval)
+
     GRIB_CHECK(_internal.grib_c_index_get_double(iid,key,a,s))
+
     result = []
     for i in range(nval):
         result.append(_internal.doubleArray_getitem(a,i))
+
+    _internal.delete_doubleArray(a)
+    _internal.delete_intp(s)
+
     return tuple(result)
 
 @require(indexid=int,key=str,val=int)
@@ -1037,6 +1087,10 @@ def grib_get_double_elements(gribid,key,indexes):
         for i in range(psize.value()):
             result.append(_internal.doubleArray_getitem(pval,i))
 
+        _internal.delete_intArray(pidx)
+        _internal.delete_doubleArray(pval)
+        _internal.delete_intp(psize)
+
         return result
 
 def grib_get_elements(gribid,key,indexes):
@@ -1134,6 +1188,12 @@ def grib_find_nearest(gribid,inlat,inlon,is_lsm = False,npoints = 1):
                 index = _internal.intArray_getitem(pidx,i), \
             ))
 
+        _internal.delete_doubleArray(poutlat)
+        _internal.delete_doubleArray(poutlon)
+        _internal.delete_doubleArray(pvalues)
+        _internal.delete_doubleArray(pdist)
+        _internal.delete_intArray(pidx)
+
         return tuple(result)
     else:
         raise ValueError("Invalid value for npoints. Expecting 1 or 4.")
