@@ -189,7 +189,7 @@ static void init_class(grib_accessor_class* c)
 
 /* END_CLASS_IMP */
 
-#define MAX_NUMBER_OF_GROUPS 450000
+#define MAX_NUMBER_OF_GROUPS 65534
 #define EFDEBUG 0
 
 static unsigned long nbits[32]={
@@ -586,6 +586,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 	long widthOfFirstOrderValues;
 	int computeGroupA=1;
 	long dataHeadersLength,widthsLength,lengthsLength,firstOrderValuesLength;
+	long decimal_scale_factor;
 
 	self->dirty=1;
 
@@ -598,6 +599,14 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 	  if (val[i] < min ) min = val[i];
 	}
 
+	if((ret = grib_get_long_internal(a->parent->h,self->decimal_scale_factor, &decimal_scale_factor))
+     != GRIB_SUCCESS)
+		return ret;
+  	decimal = grib_power(decimal_scale_factor,10);
+
+	max*=decimal;
+	min*=decimal;
+
 	if (grib_get_nearest_smaller_value(a->parent->h,self->reference_value,min,&reference_value)
         !=GRIB_SUCCESS) {
         grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
@@ -626,12 +635,6 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
      GRIB_SUCCESS)
 		return ret;
 
-	if((ret = grib_set_long_internal(a->parent->h,self->decimal_scale_factor, 0))
-     != GRIB_SUCCESS)
-		return ret;
-
-	/* decimal = grib_power(decimal_scale_factor,10) ; */
-	decimal=1.0;
   	divisor = grib_power(-binary_scale_factor,2);
 	X=grib_context_malloc_clear(a->parent->h->context,sizeof(long)*numberOfValues);
     for(i=0;i< numberOfValues;i++){
@@ -860,7 +863,6 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 				firstOrderValues[numberOfGroups] = maxA-range > 0 ? maxA-range : 0;
 				break;
 			}
-			continue;
 #endif
 			incrementGroupLengthA=startGroupLength;
 			computeGroupA=1;
@@ -1026,7 +1028,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 	grib_set_long(a->parent->h,self->N2, N2);
 	
 	if (numberOfGroups > 65535 ) {
-		extraValues=(numberOfGroups+65535)/65536;	
+		extraValues=numberOfGroups/65536;	
 		codedNumberOfGroups=numberOfGroups%65536;
 	} else {
 		extraValues=0;	
