@@ -547,7 +547,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
       return err;
   }
 
-  if ( bits_per_value == 0 ) {
+  if ( bits_per_value == 0 || (binary_scale_factor==0 && decimal_scale_factor_get!=0) ) {
     /* decimal_scale_factor is given, binary_scale_factor=0
        and bits_per_value is computed */
     binary_scale_factor=0;
