@@ -76,9 +76,14 @@ extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr,int type);
 
 #include <pthread.h>
 #define GRIB_PTHREAD_ONCE(a,b) pthread_once(a,b);
-#define GRIB_MUTEX_LOCK(a) pthread_mutex_lock(a);
+#define GRIB_MUTEX_LOCK(a) pthread_mutex_lock(a); 
 #define GRIB_MUTEX_UNLOCK(a) pthread_mutex_unlock(a);
 
+/*
+#define GRIB_MUTEX_LOCK(a) {pthread_mutex_lock(a); printf("MUTEX LOCK %p %s line %d\n",(void*)a,__FILE__,__LINE__);}
+#define GRIB_MUTEX_UNLOCK(a) {pthread_mutex_unlock(a);printf("MUTEX UNLOCK %p %s line %d\n",(void*)a,__FILE__,__LINE__);} 
+*/
+
 #else
 
 #define GRIB_PTHREAD_ONCE(a,b)
@@ -757,6 +762,8 @@ struct grib_context
     int                             no_abort;
 	int 							io_buffer_size;
 	int 							no_big_group_split;
+	int 							no_spd;
+	int 							keep_matrix;
     char*                           grib_definition_files_path;
     char*                           grib_templates_path;
     char*                           grib_concept_path;
