@@ -12,22 +12,91 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#if GRIB_PTHREADS
+static pthread_once_t once  = PTHREAD_ONCE_INIT;
+
+static pthread_mutex_t mutex_mem = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t mutex_c = PTHREAD_MUTEX_INITIALIZER;
+
+static void init() {
+	pthread_mutexattr_t attr;
+	pthread_mutexattr_init(&attr);
+	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+	pthread_mutex_init(&mutex_c,&attr);
+	pthread_mutex_init(&mutex_mem,&attr);
+	pthread_mutexattr_destroy(&attr);
+}
+
+#endif
+
+
 #if MANAGE_MEM
 
 #else
 static long cnt = 0;
 static long cntp = 0;
 
-static void  default_long_lasting_free(const grib_context* c, void* p)   {free(p);cntp--;}
-static void* default_long_lasting_malloc(const grib_context* c, size_t size) {cntp++;return malloc(size);}
-
-static void  default_buffer_free(const grib_context* c, void* p)   {free(p);cntp--;}
-static void* default_buffer_malloc(const grib_context* c, size_t size) {cntp++;return malloc(size);}
-static void* default_buffer_realloc(const grib_context* c, void* p, size_t size) {return realloc(p,size);}
-
-static void  default_free(const grib_context* c, void* p)   {free(p);cnt--;}
-static void* default_malloc(const grib_context* c, size_t size) {cnt++;return malloc(size);}
-static void* default_realloc(const grib_context* c, void* p, size_t size) {return realloc(p,size);}
+static void  default_long_lasting_free(const grib_context* c, void* p)   {
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	free(p);cntp--;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+}
+static void* default_long_lasting_malloc(const grib_context* c, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cntp++;
+	ret=malloc(size);
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	return ret;
+}
+static void  default_buffer_free(const grib_context* c, void* p)   {
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	free(p);cntp--;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+}
+static void* default_buffer_malloc(const grib_context* c, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cntp++;
+	ret=malloc(size);
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	return ret;
+}
+static void* default_buffer_realloc(const grib_context* c, void* p, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	ret=realloc(p,size);
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	return ret;
+}
+static void  default_free(const grib_context* c, void* p)   {
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	free(p);cnt--;
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+}
+static void* default_malloc(const grib_context* c, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	cnt++;
+	ret=malloc(size);
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	return ret;
+}
+static void* default_realloc(const grib_context* c, void* p, size_t size) {
+	void* ret;
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_mem);
+	ret=realloc(p,size);
+	GRIB_MUTEX_UNLOCK(&mutex_mem);
+	return ret;
+}
 #endif
 
 static size_t  default_read(const grib_context* c, void *ptr, size_t size, void *stream) {
@@ -137,6 +206,8 @@ static  grib_context default_grib_context = {
 	0,                            /* no_abort                   */
 	0,                            /* io_buffer_size             */
 	0,                            /* no_big_group_split         */
+	0,                            /* no_spd         */
+	0,                            /* keep_matrix         */
 	0,                            /* grib_definition_files_path */
 	0,                            /* grib_templates_path        */
 	0,                            /* grib_concept_path          */
@@ -205,21 +276,10 @@ static  grib_context default_grib_context = {
 #endif
 };
 
-#if GRIB_PTHREADS
-static pthread_once_t once  = PTHREAD_ONCE_INIT;
-static void init() {
-	pthread_mutexattr_t attr;
-	pthread_mutexattr_init(&attr);
-	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
-	pthread_mutex_init(&(default_grib_context.mutex),&attr);
-	pthread_mutexattr_destroy(&attr);
-
-}
-#endif
 
 grib_context* grib_context_get_default(){
 	GRIB_PTHREAD_ONCE(&once,&init);
-	GRIB_MUTEX_LOCK(&(default_grib_context.mutex));
+	GRIB_MUTEX_LOCK(&mutex_c);
 
 	if(!default_grib_context.inited)
 	{
@@ -232,9 +292,13 @@ grib_context* grib_context_get_default(){
 		const char *io_buffer_size=getenv("GRIB_API_IO_BUFFER_SIZE");
 		const char *log_stream=getenv("GRIB_API_LOG_STREAM");
 		const char *no_big_group_split=getenv("GRIB_API_NO_BIG_GROUP_SPLIT");
+		const char *no_spd=getenv("GRIB_API_NO_SPD");
+		const char *keep_matrix=getenv("GRIB_API_KEEP_MATRIX");
 		default_grib_context.inited = 1;
 		default_grib_context.io_buffer_size = io_buffer_size ? atoi(io_buffer_size) : 0;
 		default_grib_context.no_big_group_split = no_big_group_split ? atoi(no_big_group_split) : 0;
+		default_grib_context.no_spd = no_spd ? atoi(no_spd) : 0;
+		default_grib_context.keep_matrix = keep_matrix ? atoi(keep_matrix) : 1;
 		default_grib_context.write_on_fail  = write_on_fail ? atoi(write_on_fail) : 0;
 		default_grib_context.no_abort  = no_abort ? atoi(no_abort) : 0;
 		default_grib_context.debug  = debug ? atoi(debug) : 0;
@@ -242,6 +306,7 @@ grib_context* grib_context_get_default(){
 		default_grib_context.large_constant_fields = large_constant_fields ? atoi(large_constant_fields) : 0;
 		default_grib_context.ieee_packing=ieee_packing ? atoi(ieee_packing) : 0;
 		default_grib_context.grib_templates_path = getenv("GRIB_SAMPLES_PATH");
+		default_grib_context.log_stream=stderr;
 		if (!log_stream) { 
 			default_grib_context.log_stream=stderr;
 		} else if (!strcmp(log_stream,"stderr") ) {
@@ -272,7 +337,7 @@ grib_context* grib_context_get_default(){
 	}
 
 
-	GRIB_MUTEX_UNLOCK(&(default_grib_context.mutex));
+	GRIB_MUTEX_UNLOCK(&mutex_c);
 
 	return &default_grib_context;
 }
@@ -288,6 +353,7 @@ grib_context* grib_context_new(grib_context* parent)
 
 	if (!parent) parent=grib_context_get_default();
 
+	GRIB_PTHREAD_ONCE(&once,&init);
 	GRIB_MUTEX_LOCK(&(parent->mutex));
 
 	c = (grib_context*)grib_context_malloc_clear_persistent(&default_grib_context,sizeof(grib_context));
@@ -314,7 +380,7 @@ grib_context* grib_context_new(grib_context* parent)
 	
 #if GRIB_PTHREADS
 	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
-	pthread_mutex_init(&(c->mutex),&attr);
+	pthread_mutex_init(&mutex_c,&attr);
 	pthread_mutexattr_destroy(&attr);
 #endif
 
@@ -336,7 +402,8 @@ static int init_definition_files_dir(grib_context* c) {
 
 	path=c->grib_definition_files_path;
 
-	GRIB_MUTEX_LOCK(&(c->mutex));
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_c);
 
 	p=path;
 	while(*p!=':' && *p!='\0') p++;
@@ -360,7 +427,7 @@ static int init_definition_files_dir(grib_context* c) {
 		}
 	}
 
-	GRIB_MUTEX_UNLOCK(&(c->mutex));
+	GRIB_MUTEX_UNLOCK(&mutex_c);
 
 	return err;
 }
@@ -373,12 +440,15 @@ char *grib_context_full_path(grib_context* c,const char* basename)
 	grib_string_list* fullpath=0;
 	if (!c) c=grib_context_get_default();
 
-	GRIB_MUTEX_LOCK(&(c->mutex));
+	GRIB_PTHREAD_ONCE(&once,&init);
+	GRIB_MUTEX_LOCK(&mutex_c);
 
 	if(*basename == '/' || *basename ==  '.') {
+	    GRIB_MUTEX_UNLOCK(&mutex_c);
 		return (char*)basename;
 	} else {
 		if ((fullpath=grib_trie_get(c->def_files,basename))!=NULL) {
+			GRIB_MUTEX_UNLOCK(&mutex_c);
 			return fullpath->value;
 		}
 		if (!c->grib_definition_files_dir)
@@ -387,6 +457,7 @@ char *grib_context_full_path(grib_context* c,const char* basename)
 		if (err != GRIB_SUCCESS) {
 			grib_context_log(c,GRIB_LOG_ERROR,
 					"Unable to find definition files directory");
+			GRIB_MUTEX_UNLOCK(&mutex_c);
 			return NULL;
 		}
 
@@ -400,7 +471,7 @@ char *grib_context_full_path(grib_context* c,const char* basename)
 				fullpath->value=grib_context_strdup(c,full);
 				grib_trie_insert(c->def_files,basename,fullpath);
 				grib_context_log(c,GRIB_LOG_DEBUG,"Found def file %s",full);
-				GRIB_MUTEX_UNLOCK(&(c->mutex));
+				GRIB_MUTEX_UNLOCK(&mutex_c);
 				return fullpath->value;
 			}
 			dir=dir->next;
@@ -411,7 +482,7 @@ char *grib_context_full_path(grib_context* c,const char* basename)
 	/* Store missing files so we don't check for them again and again */
 	grib_trie_insert(c->def_files,basename,&grib_file_not_found);
 	/*grib_context_log(c,GRIB_LOG_ERROR,"Def file \"%s\" not found",basename);*/
-	GRIB_MUTEX_UNLOCK(&(c->mutex));
+	GRIB_MUTEX_UNLOCK(&mutex_c);
 	full[0]=0;
 	return NULL;
 }
