@@ -14,6 +14,21 @@
  ***************************************************************************/
 #include "grib_api_internal.h"
 
+#if GRIB_PTHREADS
+static pthread_once_t once  = PTHREAD_ONCE_INIT;
+static pthread_mutex_t mutex1 = PTHREAD_MUTEX_INITIALIZER;
+
+static void init() {
+    pthread_mutexattr_t attr;
+    pthread_mutexattr_init(&attr);
+    pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
+    pthread_mutex_init(&mutex1,&attr);
+    pthread_mutexattr_destroy(&attr);
+
+}
+#endif
+
+
 static grib_handle* grib_handle_new_from_file_no_multi ( grib_context* c, FILE* f,int *error );
 static grib_handle* grib_handle_new_from_file_multi ( grib_context* c, FILE* f,int *error );
 static int grib2_get_next_section ( unsigned char* msgbegin,size_t msglen,unsigned char** secbegin,size_t* seclen,int* secnum,int* err );
@@ -140,18 +155,20 @@ grib_handle* grib_new_handle ( grib_context* c )
 	g = ( grib_handle* ) grib_context_malloc_clear ( c,sizeof ( grib_handle ) );
 
 
-	if ( g == NULL )
+	if ( g == NULL ) {
 		grib_context_log ( c,GRIB_LOG_ERROR,"grib_new_handle: cannot allocate handle" );
-	else
+	} else {
+		GRIB_PTHREAD_ONCE(&once,&init);
+	    GRIB_MUTEX_LOCK(&mutex1);
 		g->context = c;
+	    GRIB_MUTEX_UNLOCK(&mutex1);
+	}
 
 	grib_context_log ( c,GRIB_LOG_DEBUG,"grib_new_handle: allocated handle %p",g );
 
 	return g;
 }
 
-
-
 static grib_handle* grib_handle_create ( grib_handle  *gl, grib_context* c,void* data, size_t buflen )
 {
 	grib_action* next = NULL;
@@ -198,7 +215,6 @@ static grib_handle* grib_handle_create ( grib_handle  *gl, grib_context* c,void*
 	grib_section_adjust_sizes ( gl->root,0,0 );
 	grib_section_post_init ( gl->root );
 
-
 	return gl;
 
 }
@@ -261,11 +277,14 @@ grib_handle* grib_handle_new_from_message_copy ( grib_context* c, const void* da
 	grib_handle *g = NULL;
 	void* copy =NULL;
 	if ( c == NULL ) c = grib_context_get_default();
+
+
 	c->handle_file_count=0;
 	c->handle_total_count=0;
 	copy = grib_context_malloc ( c,size );
-	if ( !copy )
+	if ( !copy ) {
 		return NULL;
+	}
 
 	memcpy ( copy,data,size );
 
@@ -310,9 +329,11 @@ grib_handle* grib_handle_new_from_partial_message ( grib_context* c,void* data,
 grib_handle* grib_handle_new_from_message ( grib_context* c,void* data, size_t buflen )
 {
 	grib_handle  *gl = NULL;
+	grib_handle  *h = NULL;
 	if ( c == NULL ) c = grib_context_get_default();
 	gl = grib_new_handle ( c );
-	return grib_handle_create ( gl,  c, data,  buflen );
+	h=grib_handle_create ( gl,  c, data,  buflen );
+	return h;
 }
 
 
@@ -951,8 +972,14 @@ int grib_get_message_copy ( grib_handle* h ,  void* message,size_t *len )
 
 int grib_get_message ( grib_handle* h,const void** msg,size_t* size )
 {
+	long totalLength=0;
+	int ret=0;
 	*msg  =  h->buffer->data;
 	*size =  h->buffer->ulength;
+
+	ret=grib_get_long(h,"totalLength",&totalLength);
+	if (!ret) *size=totalLength;
+
 	if ( h->context->gts_header_on && h->gts_header )
 	{
 		char strbuf[10];
