@@ -83,12 +83,15 @@ static int read_the_rest(reader* r,size_t message_length,unsigned char* tmp, int
 
 #define CHECK_TMP_SIZE(a) if(sizeof(tmp)<(a)) { fprintf(stderr,"%s:%d sizeof(tmp)<%s %d<%d\n", __FILE__,__LINE__,#a,(int)sizeof(tmp),(int)(a)); return GRIB_INTERNAL_ARRAY_TOO_SMALL; }
 
+
 #define UINT3(a,b,c) (size_t)((a<<16) + (b<<8) + c);
 
 static int read_GRIB(reader* r)
 {
 	/* unsigned char tmp[16368];  */
-	unsigned char tmp[700368]; /* Should be enough */
+	/* unsigned char tmp[700368]; */
+	/*unsigned char tmp[8388608];   Should be enough */
+	unsigned char tmp[1048576];  
 	size_t length = 0;
 	size_t total_length = 0;
 	long edition = 0;
@@ -228,6 +231,7 @@ static int read_GRIB(reader* r)
 					}
 
 					/* Read section 3 */
+					CHECK_TMP_SIZE(sec1len + sec2len + sec3len + 4 + 3 );
 					if((r->read(r->read_data,tmp+i,sec3len-3,&err) != sec3len-3) || err)
 						return err;
 					i += sec3len-3;
@@ -846,6 +850,7 @@ int grib_read_any_from_file(grib_context* ctx,FILE* f,void* buffer,size_t* len)
 	int         err;
 	user_buffer u; 
 	reader      r; 
+	off_t offset;
 
 	u.user_buffer  = buffer;
 	u.buffer_size  = *len;
@@ -859,8 +864,16 @@ int grib_read_any_from_file(grib_context* ctx,FILE* f,void* buffer,size_t* len)
 	r.alloc        = &user_provider_buffer;
 	r.headers_only = 0;
 
+
+	offset=ftello(f);
+
 	err            = read_any(&r,1,GRIB_API_READS_BUFR);
 
+	if (err==GRIB_BUFFER_TOO_SMALL) {
+	   if (fseeko(f,offset,SEEK_SET)) 
+	   		err=GRIB_IO_PROBLEM;
+	}
+
 	*len           = u.message_size;
 
 	return err;
