@@ -382,6 +382,7 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 	double laplacianOperator;
 	int packingTypeIsSet=0;
 	int setSecondOrder=0;
+    size_t slen=17;
 
 	static grib_util_packing_spec default_packing_spec = {0, };
 
@@ -389,6 +390,15 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 		packing_spec = &default_packing_spec;
 	}
 
+	/* Get edition number from input handle */
+	if((*err = grib_get_long(h,"edition",&editionNumber)) != 0)
+	{
+		grib_context* c=grib_context_get_default();
+		if (c->write_on_fail) 
+			grib_write_message(h,"error.grib","w");
+		return NULL;
+	}
+
 	if (packing_spec->deleteLocalDefinition) {
 		/* TODO: not working for grib2 */
 		SET_LONG_VALUE("deleteLocalDefinition",1);
@@ -497,6 +507,7 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 		if (h->context->debug==-1) 
 			printf("GRIB_API DEBUG grib_util: done grib_set_double_array \n");
 
+
 		/* convert to second_order if not constant field */
         if (setSecondOrder ) {
                 size_t slen=17;
@@ -519,7 +530,27 @@ grib_handle* grib_util_set_spec(grib_handle* h,
                     }
                 }
                 if (!constant) {
-                    grib_set_string(h,"packingType","grid_second_order",&slen);
+					if (editionNumber == 1 ) {
+						long numberOfGroups;
+						grib_handle* htmp=grib_handle_clone(h);
+
+						slen=17;
+						grib_set_string(htmp,"packingType","grid_second_order",&slen);
+						grib_get_long(htmp,"numberOfGroups",&numberOfGroups);
+						/* GRIBEX is not able to decode overflown numberOfGroups with SPD */
+						if (numberOfGroups>65534 && h->context->no_spd ) {
+							slen=24;
+							grib_set_string(h,"packingType","grid_second_order_no_SPD",&slen);
+							grib_handle_delete(htmp);
+						} else {
+							grib_handle_delete(h);
+							h=htmp;
+						}
+					} else {
+						slen=17;
+						grib_set_string(h,"packingType","grid_second_order",&slen);
+						grib_set_double_array(h,"values",data_values,data_values_count);
+					}
 				} else {
 					if (h->context->gribex_mode_on) {
 						h->context->gribex_mode_on=0;
@@ -532,12 +563,6 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 		return h;
 	}
 
-	/* Get edition number from input handle */
-	if((*err = grib_get_long(h,"editionNumber",&editionNumber)) != 0)
-	{
-		return NULL;
-	}
-
 
 	switch(spec->grid_type) {
 
@@ -716,15 +741,9 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 			{
 				SET_STRING_VALUE("packingType", "spectral_complex");
 				packingTypeIsSet=1;
-				if (!strcmp(input_packing_type,"spectral_simple")) {
-					SET_LONG_VALUE("JS", 0);
-					SET_LONG_VALUE("KS", 0);
-					SET_LONG_VALUE("MS", 0);
-				} else {
-					SET_LONG_VALUE("JS", 20);
-					SET_LONG_VALUE("KS", 20);
-					SET_LONG_VALUE("MS", 20);
-				}
+				SET_LONG_VALUE("JS", 20);
+				SET_LONG_VALUE("KS", 20);
+				SET_LONG_VALUE("MS", 20);
 				if (packing_spec->packing == GRIB_UTIL_PACKING_USE_PROVIDED && editionNumber==2 ) {
 					SET_LONG_VALUE("computeLaplacianOperator", 1);
 				} else if ((!(*err) && strcmp(input_grid_type,"sh")) || packing_spec->computeLaplacianOperator ) {
@@ -797,17 +816,21 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 	if (!strcmp(input_packing_type,"grid_simple_matrix")) {
 		long numberOfDirections,numberOfFrequencies;
 		int ret;
-		SET_STRING_VALUE("packingType","grid_simple_matrix");
-		ret=grib_get_long(h,"numberOfDirections",&numberOfDirections);
-		if (!ret) {
+		if (h->context->keep_matrix) {
+			SET_STRING_VALUE("packingType","grid_simple_matrix");
 			ret=grib_get_long(h,"numberOfDirections",&numberOfDirections);
-			SET_LONG_VALUE("NC1",numberOfDirections);
-			ret=grib_get_long(h,"numberOfFrequencies",&numberOfFrequencies);
-			SET_LONG_VALUE("NC2",numberOfFrequencies);
-			SET_LONG_VALUE("physicalFlag1",1);
-			SET_LONG_VALUE("physicalFlag2",2);
-			SET_LONG_VALUE("NR",1);
-			SET_LONG_VALUE("NC",1);
+			if (!ret) {
+				ret=grib_get_long(h,"numberOfDirections",&numberOfDirections);
+				SET_LONG_VALUE("NC1",numberOfDirections);
+				ret=grib_get_long(h,"numberOfFrequencies",&numberOfFrequencies);
+				SET_LONG_VALUE("NC2",numberOfFrequencies);
+				SET_LONG_VALUE("physicalFlag1",1);
+				SET_LONG_VALUE("physicalFlag2",2);
+				SET_LONG_VALUE("NR",1);
+				SET_LONG_VALUE("NC",1);
+			}
+		} else {
+			SET_STRING_VALUE("packingType","grid_simple");
 		}
 	}
 
@@ -883,6 +906,7 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 	{
 		FILE* ferror;
 		long i,count;
+		grib_context* c=grib_context_get_default();
 
 		ferror=fopen("error.data","w");
 		count=0;
@@ -895,14 +919,25 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 		}
 		fprintf(ferror,"%g }",data_values[data_values_count-1]);
 		fclose(ferror);
-		grib_write_message(outh,"error.grib","w");
+		if (c->write_on_fail) 
+			grib_write_message(outh,"error.grib","w");
 		goto cleanup;
 	}
 	/* grib_write_message(outh,"h.grib","w"); */
+	/* if the field is empty GRIBEX is packing as simple*/
+	/*
+	if (!strcmp(input_packing_type,"grid_simple_matrix")) {
+		long numberOfValues;
+		grib_get_long(outh,"numberOfValues",&numberOfValues);
+		if (numberOfValues==0)  {
+			slen=11;
+			grib_set_string(outh,"packingType","grid_simple",&slen);
+		}
+	}
+	*/
 
 		/* convert to second_order if not constant field */
         if (setSecondOrder ) {
-                size_t slen=17;
                 int ii=0;
                 int constant=1;
 				double missingValue=0;
@@ -922,7 +957,27 @@ grib_handle* grib_util_set_spec(grib_handle* h,
                     }
                 }
                 if (!constant) {
-                    grib_set_string(outh,"packingType","grid_second_order",&slen);
+					if (editionNumber == 1 ) {
+						long numberOfGroups;
+						grib_handle* htmp=grib_handle_clone(outh);
+
+						slen=17;
+						grib_set_string(htmp,"packingType","grid_second_order",&slen);
+						grib_get_long(htmp,"numberOfGroups",&numberOfGroups);
+						/* GRIBEX is not able to decode overflown numberOfGroups with SPD */
+						if (numberOfGroups>65534 && outh->context->no_spd) {
+							slen=24;
+							grib_set_string(outh,"packingType","grid_second_order_no_SPD",&slen);
+							grib_handle_delete(htmp);
+						} else {
+							grib_handle_delete(outh);
+							outh=htmp;
+						}
+					} else {
+						slen=17;
+						grib_set_string(outh,"packingType","grid_second_order",&slen);
+						grib_set_double_array(outh,"values",data_values,data_values_count);
+					}
 				} else {
 					if (outh->context->gribex_mode_on) {
 						outh->context->gribex_mode_on=0;
@@ -932,9 +987,8 @@ grib_handle* grib_util_set_spec(grib_handle* h,
 				}
         }
 
-
 	if(packing_spec->editionNumber && packing_spec->editionNumber!=editionNumber) 
-		grib_set_long(outh,"editionNumber", packing_spec->editionNumber);
+		grib_set_long(outh,"edition", packing_spec->editionNumber);
 
 	if (h->context->debug==-1) 
 		printf("GRIB_API DEBUG: grib_util_set_spec end\n");
