@@ -20,7 +20,13 @@ cat  > $$_f <<EOF
 	}
 EOF
 
-localDefinitions=`ls ${def_dir}/grib1/local.98.*def | awk 'BEGIN {FS=".";} {print $3;}' | grep -v def | xargs` 
+localDefinitions=`find ${def_dir}/grib1/ -name 'local.98.*def' -printf '%f\n' |\
+                awk 'BEGIN {FS=".";} {print $3;}' |\
+                grep -v def |\
+                sed '/245/d' |\
+                sed '/244/d' |\
+                sed '/192/d' |\
+                xargs`
 
 for l1 in $localDefinitions
 do
