@@ -70,7 +70,7 @@ int tolerance_factor=1;
 GRIB_INLINE static double compare_double_absolute(double *a,double *b,double *err) {
   double ret=0;
   double d=fabs(*a-*b);
-  if (ret > *err) {
+  if (d > *err) {
   	ret=d;
   }
   return ret;
